* Sgb: Grain boundary surface energy
* Spm: Surface energy for matrix/precipitate interface
* deltGv: Driving force for nucleation
* theta: Angle of contact precipitate/grain boundary
* Nv: Number density of nucleation sites
* ETp, ETgb, ETpm: Shape factors
* Rstar: Critical radius for nucleation
* Gstar: Activation energy for nucleation 

      program coars2

      implicit none

      integer nbradi

      parameter (nbradi=10000)
            
      double precision Sgb, Spm, Q, D 
      double precision Nv, Ntotn, Ntotf, Nvinit, I
      dimension Nv(1:2), Ntotn(1:2), Ntotf(1:2), Nvinit(1:2), I(1:2)
      double precision Cpm, Cmpinf, Cbulk, Cbulk0
      double precision Xpm, Xmpinf, Xbulk
      double precision V, deltCt
      double precision Temp, Vmol, Pi
      double precision Gm, Gpm, np
      double precision delGm, delGv, f, averad, Ntot
      double precision maxtim, dt, nbstpt, t, addtim, dt2
      double precision radin1, radin2, radin3, radin4
      double precision radin5, radin6, radin7, radin8
      double precision rad, N, Invel, Cmpr, deltC, deltV, Invelc
      dimension rad(1:2,1:nbradi), N(1:2,0:nbradi), Invel(1:2,1:nbradi)
      dimension Cmpr(1:2,0:nbradi), deltC(1:2,1:nbradi)
      dimension deltV(1:2,1:nbradi),Invelc(1:2,1:nbradi) 
          
      integer j, k, l, m, p, r, s, u, w, count, count2, ii, xx, yy
      integer jj, kk, ll, mm, nn, oo, pp, qq, rr, ss, tt, uu, vv, ww 
      integer nbcoun

      parameter (Q=80.D3, Sgb=0.3D0, Spm=0.2D0)
      parameter (Temp=923.D0, Vmol=5.84D-6, D=1.84D-11)
      parameter (Gm=-6.6656242926D7, Gpm=-6.6740668227D7)
      parameter (np=27.19D0, V=1.D0)
      parameter (Xbulk=0.005D0, Xpm=0.25D0, Xmpinf=3.9D-4) 
      parameter (dt=1.D-3, maxtim=13.D3)      
     
      Pi=dacos(-1.D0)
      
      nbstpt=Int(maxtim/dt)
      Cpm=Xpm/Vmol
      Cmpinf=Xmpinf/Vmol
      Cbulk0=Xbulk/Vmol
      Cbulk=Cbulk0
      Cmpr(1,0)=Cmpinf
      Cmpr(2,0)=Cmpinf
      Nvinit(1)=1.D14
      Nvinit(2)=1.D24
      Nv(1)=Nvinit(1)
      Nv(2)=Nvinit(2)
      Ntotf(1)=0.D0
      Ntotf(2)=0.D0
      rad(1,1)=0.D0
      rad(2,1)=0.D0
      N(1,0)=Nvinit(1)
      N(2,0)=Nvinit(2)
      count=0

      open (12, file='radgb')
      open (14, file='Ngb')
      open (13, file='radint')
      open (15, file='Nint')
      open (16, file='Cb')
     
      do 10 j=0,nbstpt
         t=(dble(j))*dt
         deltCt=0.D0
         averad=0.D0
         Ntot=0.D0
        
         if (j .eq. 0) then
            write (12,900) t, rad(1,1)
            write (14,900) t, N(1,0)
            write (13,900) t, rad(2,1)
            write (15,900) t, N(2,0)
            write (16,900) t, Cbulk, averad, Ntot
            goto 10
         endif
                

         if (((N(1,j-1) .le. 1.D3) .or. (Nv(1) .eq. 0.D0))
     &    .and.((N(2,j-1) .le. 1.D3) .or. (Nv(2) .eq. 0.D0))) then
            m=j
            goto 40
         elseif ((N(1,j-1) .le. 1.D3) .or. (Nv(1) .eq. 0.D0)) then
            s=j
            ss=2
            tt=1
            goto 70
         elseif ((N(2,j-1) .le. 1.D3) .or. (Nv(2) .eq. 0.D0)) then
            s=j
            ss=1
            tt=2
            goto 70
         endif
               

         f=(Cbulk-Cmpinf)/(Cbulk0-Cmpinf)
         delGm=((Gpm-Gm)/np)*f
         delGv=delGm/Vmol

                                    
         call nucl_rate_gb (Spm, Sgb, delGv, Nv(1), Temp, Q, I(1))
         N(1,j)=I(1)*dt 
         rad(1,j)=(-2.D0*Spm)/(delGv)

         call nucl_rate_int (Spm, delGv, Nv(2), Temp, Q, I(2))
         N(2,j)=I(2)*dt
         rad(2,j)=(-2.D0*Spm)/(delGv)

         do 30 jj=1,2
            Ntotn(jj)=Ntotf(jj)+N(jj,j)
            if (Ntotn(jj) .ge. Nvinit(jj)) then
               N(jj,j)=Nvinit(jj)-Ntotf(jj)
            else 
               Ntotf(jj)=Ntotn(jj)   
            endif
 30      continue

         if (j .eq. 1) then

            do 35 kk=1,2
               deltV(kk,j)=N(kk,j)*V*(4.D0/3.D0)*Pi*rad(kk,j)*rad(kk,j)
     &          *rad(kk,j)            
               call Gibbs_Thomson (Xmpinf, Xpm, Spm, Vmol, rad(kk,j), 
     &          Temp, Cmpr(kk,j))
               call delt_compo (deltV(kk,j), V, Cpm, Cmpr(kk,j), 
     &          deltC(kk,j))
               averad=averad+N(kk,j)*rad(kk,j)
               Ntot=Ntot+N(kk,j)
 35         continue

            Cbulk=Cbulk+deltC(1,j)+deltC(2,j)

            averad=averad/Ntot
            goto 11
         endif

         do 20 l=1,j-1

               call Interface_velocity_spherical (Cbulk, D, rad(1,l), 
     &          Cpm, Cmpr(1,l), Invel(1,l))
               call change_vol (N(1,l), V, rad(1,l), Invel(1,l), dt, 
     &          deltV(1,l))
               call change_rad_precip (rad(1,l), Invel(1,l), dt, 
     &          N(1,l), V, deltV(1,l), radin1)
               rad(1,l)=radin1
               call Gibbs_Thomson (Xmpinf, Xpm, Spm, Vmol, rad(1,l),
     &          Temp, Cmpr(1,l))
               call delt_compo (deltV(1,l), V, Cpm, Cmpr(1,l), 
     &          deltC(1,l))
               averad=averad+N(1,l)*rad(1,l)
               Ntot=Ntot+N(1,l)

               call Interface_velocity_spherical (Cbulk, D, rad(2,l), 
     &          Cpm, Cmpr(2,l), Invel(2,l))
               call change_vol (N(2,l), V, rad(2,l), Invel(2,l), dt, 
     &          deltV(2,l))
               call change_rad_precip (rad(2,l), Invel(2,l), dt, 
     &          N(2,l), V, deltV(2,l), radin1)
               rad(2,l)=radin1
               call Gibbs_Thomson (Xmpinf, Xpm, Spm, Vmol, rad(2,l),
     &          Temp, Cmpr(2,l))
               call delt_compo (deltV(2,l), V, Cpm, Cmpr(2,l), 
     &          deltC(2,l))
               averad=averad+N(2,l)*rad(2,l)
               Ntot=Ntot+N(2,l)

            deltCt=deltCt+deltC(1,l)+deltC(2,l)
 20      continue

         do 55 mm=1,2
            deltV(mm,j)=N(mm,j)*V*(4.D0/3.D0)*Pi*rad(mm,j)*rad(mm,j)
     &       *rad(mm,j)
            call delt_compo (deltV(mm,j), V, Cpm, Cmpr(mm,j),
     &       deltC(mm,j))
            averad=averad+N(mm,j)*rad(mm,j)
            Ntot=Ntot+N(mm,j)
 55      continue

         Cbulk=Cbulk+deltCt+deltC(1,j)+deltC(2,j)

         averad=averad/Ntot
         
 11      write (12,900) t, (rad(1,ii),ii=1,j)
         write (14,900) t, (N(1,ii),ii=1,j)
         write (13,900) t, (rad(2,ii),ii=1,j)        
         write (15,900) t, (N(2,ii),ii=1,j)
         write (16,900) t, Cbulk, averad, Ntot

         Nv(1)=Nv(1)-N(1,j)
         Nv(2)=Nv(2)-N(2,j)

 10      continue

* If Nv(1)=0 & Nv(2)=0 in the same step 

 40      do 50 p=m,nbstpt
            t=(dble(p))*dt
            count=count+1
            deltCt=0.D0
            averad=0.D0
            Ntot=0.D0

            do 60 r=1, m-1

                  call Interface_velocity_spherical (Cbulk, D, 
     &             rad(1,r), Cpm, Cmpr(1,r), Invel(1,r))
                  call change_vol (N(1,r), V, rad(1,r), Invel(1,r),
     &             dt, deltV(1,r))
                  call change_rad_precip (rad(1,r), Invel(1,r), dt,
     &             N(1,r), V, deltV(1,r), radin2)
                  rad(1,r)=radin2
                  call Gibbs_Thomson (Xmpinf, Xpm, Spm, Vmol, rad(1,r),
     &             Temp, Cmpr(1,r))
                  call delt_compo (deltV(1,r), V, Cpm, Cmpr(1,r), 
     &             deltC(1,r))
                  averad=averad+N(1,r)*rad(1,r)
                  Ntot=Ntot+N(1,r)
 
                  call Interface_velocity_spherical (Cbulk, D, 
     &             rad(2,r), Cpm, Cmpr(2,r), Invel(2,r))
                  call change_vol (N(2,r), V, rad(2,r), Invel(2,r),
     &             dt, deltV(2,r))
                  call change_rad_precip (rad(2,r), Invel(2,r), dt,
     &             N(2,r), V, deltV(2,r), radin2)
                  rad(2,r)=radin2
                  call Gibbs_Thomson (Xmpinf, Xpm, Spm, Vmol, rad(2,r),
     &             Temp, Cmpr(2,r))
                  call delt_compo (deltV(2,r), V, Cpm, Cmpr(2,r), 
     &             deltC(2,r))
                  averad=averad+N(2,r)*rad(2,r)
                  Ntot=Ntot+N(2,r)

               deltCt=deltCt+deltC(1,r)+deltC(2,r)
 60         continue

            Cbulk=Cbulk+deltCt

            averad=averad/Ntot

            if (count .eq. 100001) then
               write (12,900) t, (rad(1,ii),ii=1,m-1)
               write (14,900) t, (N(1,ii),ii=1,m-1)
               write (13,900) t, (rad(2,ii),ii=1,m-1)
               write (15,900) t, (N(2,ii),ii=1,m-1)
               write (16,900) t, Cbulk, averad, Ntot
               count=1
            endif

 50      continue
         goto 500

* If Nv(tt)=0 in first with tt=1 or 2
 70      do 75 oo=s,nbstpt
            t=(dble(oo))*dt
*            count=count+1
            deltCt=0.D0
            averad=0.D0
            Ntot=0.D0
            
            if (oo .gt. s) then
               Nv(ss)=Nv(ss)-N(ss,oo-1)
            endif

            if (N(ss,oo-1) .le. 1.D3) then
               w=oo
               goto 90
            endif

            f=(Cbulk-Cmpinf)/(Cbulk0-Cmpinf)
            delGm=((Gpm-Gm)/np)*f
            delGv=delGm/Vmol

            call nucl_rate_int (Spm, delGv, Nv(ss), Temp, Q, I(ss))
            N(ss,oo)=I(ss)*dt
            rad(ss,oo)=(-2.D0*Spm)/(delGv)

            Ntotn(ss)=Ntotf(ss)+N(ss,oo)

            if (Ntotn(ss) .ge. Nvinit(ss)) then
               N(ss,oo)=Nvinit(ss)-Ntotf(ss)
            else 
               Ntotf(ss)=Ntotn(ss)   
            endif

            do 80 qq=1,s-1
               call Interface_velocity_spherical (Cbulk, D, rad(tt,qq), 
     &          Cpm, Cmpr(tt,qq), Invel(tt,qq))
               call change_vol (N(tt,qq), V, rad(tt,qq), Invel(tt,qq),
     &          dt, deltV(tt,qq))
               call change_rad_precip (rad(tt,qq), Invel(tt,qq), dt, 
     &          N(tt,qq), V, deltV(tt,qq), radin3)
               rad(tt,qq)=radin3
               call Gibbs_Thomson (Xmpinf, Xpm, Spm, Vmol, rad(tt,qq),
     &          Temp, Cmpr(tt,qq))
               call delt_compo (deltV(tt,qq), V, Cpm, Cmpr(tt,qq), 
     &          deltC(tt,qq))
               deltCt=deltCt+deltC(tt,qq)
               averad=averad+N(tt,qq)*rad(tt,qq)
               Ntot=Ntot+N(tt,qq)
 80         continue
            
            do 85 rr=1,oo-1
               call Interface_velocity_spherical (Cbulk, D, rad(ss,rr), 
     &          Cpm, Cmpr(ss,rr), Invel(ss,rr))
               call change_vol (N(ss,rr), V, rad(ss,rr), Invel(ss,rr),
     &          dt, deltV(ss,rr))
               call change_rad_precip (rad(ss,rr), Invel(ss,rr), dt, 
     &          N(ss,rr), V, deltV(ss,rr), radin4)
               rad(ss,rr)=radin4
               call Gibbs_Thomson (Xmpinf, Xpm, Spm, Vmol, rad(ss,rr),
     &          Temp, Cmpr(ss,rr))
               call delt_compo (deltV(ss,rr), V, Cpm, Cmpr(ss,rr), 
     &          deltC(ss,rr))
               deltCt=deltCt+deltC(ss,rr)
               averad=averad+N(ss,rr)*rad(ss,rr)
               Ntot=Ntot+N(ss,rr)
 85         continue
            
         deltV(ss,oo)=N(ss,oo)*V*(4.D0/3.D0)*Pi*rad(ss,oo)*rad(ss,oo)
     &       *rad(ss,oo)
         call delt_compo (deltV(ss,oo), V, Cpm, Cmpr(ss,oo), 
     &     deltC(ss,oo))
         averad=averad+N(ss,oo)*rad(ss,oo)
         Ntot=Ntot+N(ss,oo)

         Cbulk=Cbulk+deltCt+deltC(ss,oo)
         
         averad=averad/Ntot

*         if (count .eq. 101) then
            if (ss .eq. 2) then
               write (12,900) t, (rad(1,ii),ii=1,s-1)
               write (13,900) t, (rad(2,ii),ii=1,oo)
               write (15,900) t, (N(2,ii),ii=1,oo)
               write (16,900) t, Cbulk, averad, Ntot
            else
               write (12,900) t, (rad(1,ii),ii=1,oo)
               write (14,900) t, (N(1,ii),ii=1,oo)
               write (13,900) t, (rad(2,ii),ii=1,s-1)
               write (16,900) t, Cbulk, averad, Ntot
            endif
*            count=1
*         endif   
         
 75   continue
      goto 500

* When Nv(ss) becomes equal to 0 too
 90   xx=w
      
      dt2=dt
      count2=0
      nbcoun=11

      do 95 uu=w,nbstpt
         t=(dble(uu))*dt2
         count=count+1
         deltCt=0.D0
         averad=0.D0
         Ntot=0.D0


         do 100 vv=1,s-1
            call Interface_velocity_spherical (Cbulk, D, rad(tt,vv),
     &       Cpm, Cmpr(tt,vv), Invel(tt,vv))
            call change_vol (N(tt,vv), V, rad(tt,vv), Invel(tt,vv),
     &       dt2, deltV(tt,vv))
            call change_rad_precip (rad(tt,vv), Invel(tt,vv), dt2,
     &       N(tt,vv), V, deltV(tt,vv), radin5)
            rad(tt,vv)=radin5
            call Gibbs_Thomson (Xmpinf, Xpm, Spm, Vmol, rad(tt,vv),
     &       Temp, Cmpr(tt,vv))
            call delt_compo (deltV(tt,vv), V, Cpm, Cmpr(tt,vv), 
     &       deltC(tt,vv))
            deltCt=deltCt+deltC(tt,vv)
            averad=averad+N(tt,vv)*rad(tt,vv)   
            Ntot=Ntot+N(tt,vv)
 100     continue

         do 110 ww=w-1,1,-1
            call Interface_velocity_spherical (Cbulk, D, rad(ss,ww),
     &       Cpm, Cmpr(ss,ww), Invel(ss,ww))
            call change_vol (N(ss,ww), V, rad(ss,ww), Invel(ss,ww),
     &       dt2, deltV(ss,ww))
            averad=averad+N(ss,ww)*rad(ss,ww)
            Ntot=Ntot+N(ss,ww)
            call change_rad_precip (rad(ss,ww), Invel(ss,ww), dt2,
     &       N(ss,ww), V, deltV(ss,ww), radin6)
            rad(ss,ww)=radin6
            if (rad(ss,ww) .eq. 0.D0) then
               w=w-1
               goto 105
            endif
            call Gibbs_Thomson (Xmpinf, Xpm, Spm, Vmol, rad(ss,ww),
     &       Temp, Cmpr(ss,ww))
 105        call delt_compo (deltV(ss,ww), V, Cpm, Cmpr(ss,ww), 
     &       deltC(ss,ww))
            deltCt=deltCt+deltC(ss,ww) 

 110     continue

         Cbulk=Cbulk+deltCt
         
         averad=averad/Ntot
         
         if (count .eq. nbcoun) then
            if (ss .eq. 2) then
               write (12,900) t, (rad(1,ii),ii=1,s-1)
               write (13,900) t, (rad(2,ii),ii=1,xx-1)
               write (16,900) t, Cbulk, averad, Ntot
            else
               write (12,900) t, (rad(1,ii),ii=1,xx-1)
               write (13,900) t, (rad(2,ii),ii=1,s-1)
               write (16,900) t, Cbulk, averad, Ntot
            endif
            count=1
            count2=count2+1
         endif

*         dt2=(1.D2/nbstpt)*((t-(dble(w)*dt))*dt2)+dt
         
*         if ((Cbulk-Cmpinf) .le. 1.D-1) then
*            dt=1.D0
*            nbstpt=addtim/dt
*            nbcoun=10001
*         endif

         if (count2 .eq. 1000) then
            dt2=1.D-1
            nbstpt=360000000
            nbcoun=10001
         endif

 95      continue

                     
 500     close (12)
         close (13)
         close (14)
         close (15)
         close (16)

 900     format (f15.4,2X,10000(d20.10,2X))
         
 999     end
         
*                                ********

      
      subroutine nucl_rate_gb (Spm, Sgb, deltGv, Nv, T, Q, I)
      
      implicit none

      double precision Spm, Sgb, deltGv, Nv, T, Q
      double precision I
      double precision ETp, ETgb, ETpm 
      double precision Gstar
      double precision theta, Afreq, Nafreq 
      double precision Pi, h, R, Avog

      Pi=dacos(-1.D0)
      R=8.31451D0
      Avog=6.0221367D23
      h=6.6260755D-34

      theta=dacos(Sgb/(2.D0*Spm))
     
* The shape factors
      ETp=2.D0*Pi*(2.D0-(3.D0*dcos(theta))+(dcos(theta)*dcos(theta)))/
     &  3.D0
      ETgb=Pi*(dsin(theta)*dsin(theta))
      ETpm=4.D0*Pi*(1.D0-dcos(theta))

* Activity energy
      Gstar=(4.D0/27.D0)*((ETpm*Spm-ETgb*Sgb)**(3.D0))
      Gstar=Gstar/(ETp*ETp*deltGv*deltGv)
      Gstar=Gstar*Avog

* Attempt frequency
      Afreq=(R*T)/(Avog*h)

* Nv*Afreq
      Nafreq=Nv*Afreq

* Nucleation rate
      I=Nafreq*(dexp(-Gstar/(R*T)))*(dexp(-Q/(R*T)))

      end



*                                ********

      
      subroutine nucl_rate_int (Spm, deltGv, Nv, T, Q, I)
      
      implicit none

      double precision Spm, deltGv, Nv, T, Q
      double precision I 
      double precision Gstar
      double precision Afreq, Nafreq 
      double precision Pi, h, R, Avog

      Pi=dacos(-1.D0)
      R=8.31451D0
      Avog=6.0221367D23
      h=6.6260755D-34
     
* Activity energy
      Gstar=(16.D0/3.D0)*Pi*(Spm*Spm*Spm)
      Gstar=Gstar/(deltGv*deltGv)
      Gstar=Gstar*Avog

* Attempt frequency
      Afreq=(R*T)/(Avog*h)

* Nv*Afreq
      Nafreq=Nv*Afreq

* Nucleation rate
      I=Nafreq*(dexp(-Gstar/(R*T)))*(dexp(-Q/(R*T)))

      end



*                               ********

* Subroutine Gibbs_Thomson gives the mole fraction of solute in the matrix in 
* equilibrium with the precipitate, function of the radius of the precipitate.  


      subroutine Gibbs_Thomson (Xmps, Xpms, Smps, Vmols, rads, 
     & Temps, Cmprs)

      implicit none
      
      double precision Xmps, Xpms, Smps, Vmols, rads, Temps
      double precision Cmprs
      double precision R

      R=8.31451D0

      Cmprs=(2.D0*Vmols*Smps)/(R*Temps*(Xpms-Xmps))
      Cmprs=dexp(Cmprs/rads)
      Cmprs=Xmps*Cmprs
      Cmprs=Cmprs/Vmols

      end


*                              ********

  
* Subroutine Interface_velocity_spherical 

      
      subroutine Interface_velocity_spherical (Cbulks, Ds, rads, Cpms,
     & Cmprs, Intvs)

      implicit none
     
      double precision Cbulks, Ds, rads, Cpms, Cmprs
      double precision Intvs
      double precision supsat
      
      supsat=(Cbulks-Cmprs)/(Cpms-Cmprs)
      Intvs=(supsat*Ds)
      Intvs=Intvs/(rads)
      
      end


*                              ********

* Subroutine change_vol calculates the volume change of the precipitates in V
 


      subroutine change_vol (Ns, Vs, rads, Intvs, dts, deltVs)
     
      implicit none

      double precision Ns, Vs, rads, Intvs, dts
      double precision Pi, deltVs

      Pi=dacos(-1.D0)
      
      deltVs=Ns*Vs*(4.D0)*Pi*rads*rads*Intvs*dts

      end


*                              ********

* Subroutine change_rad_precip gives the new precipitate radius after
* dissolution (step by step)


      subroutine change_rad_precip (rbdiss, Intvs, dts, Ns, Vs,
     & deltVs, radiss)

      implicit none
     
      double precision rbdiss, Intvs, dts, Ns, Vs, Pi
      double precision deltVs, radiss

      Pi=dacos(-1.D0)

      radiss=rbdiss*rbdiss*rbdiss
      radiss=radiss+((3.D0)*rbdiss*rbdiss*Intvs*dts)

      if (radiss .le. 0.D0) then
         radiss=0.D0
         deltVs=Ns*Vs*(4.D0/3.D0)*Pi*(rbdiss*rbdiss*rbdiss)
         else
         radiss=radiss**((1.D0)/(3.D0))
      endif
   
      end


*                              ********

* Subroutine delt_compo, assuming that the molar volumes are equal.


      subroutine delt_compo (deltVs, Vs, Ca, Cb, deltCs)
      
      implicit none
                     
      double precision deltVs, Vs, Ca, Cb
      double precision deltCs

      deltCs=-(deltVs/Vs)*(Ca-Cb)

      end


